######################################################################
# Utils for generating tables and figures in manuscript.
######################################################################
myutil <- new.env()

myutil$extractdf <- function(
        results,  # experi results
        prefix,   # prefix of modellers to be extracted from results
        postfix,  # postfix of them
        label,    # label of modellers to be in plot legend
        varname   # compared variables (such as ntrees, nvars)
)
{
    # Extract from results with the prefix and postfix in column
    # modeller
    
    datadf <- NULL
    middle <- "[[:digit:]]+"
    params <- data.frame(prefix, middle, postfix, label)
    extractdf <- function(param)
    {
        df <- subset(results,
                     grepl(paste0(param[1], param[2], param[3]),
                           modeller))
        vv <- as.integer(gsub(param[3], "",
                              gsub(param[1], "", df$modeller)))
        df <- cbind(df, vv)
        colnames(df)[ncol(df)] <- varname
        df$modeller <- ifelse(param[4]=="rf",
                              "randomForest",
                              param[4])
        return(df)
    }
    dflist <- apply(params, 1, extractdf)
    for (i in 1:length(dflist))
        datadf <- rbind(datadf, dflist[[i]])
    
    return(datadf)
}

myutil$compareCharts <- function(
  results,  # experi results
  prefix,   # prefix of modellers to be extracted from results
  postfix,  # postfix of them
  label,    # label of modellers to be in plot legend
  varname,  # compared variables (such as ntrees, nvars)
  measures, # evaluation measures (acc, cor, str, elap, mem)
  ylimts=rep(NA, length(measures)), # range of y-axis
  facets=TRUE # use facets to display plots
)
{
  require(ggplot2)
  require(gridExtra)
  require(scales)
  
  # extract related data for comparison.
  datadf <- myutil$extractdf(results,
                             prefix,
                             postfix,
                             label,
                             varname)
  
  # plot
  colorlist <- c("wsrf (wsrf_c = wsrf_t)"="red",
                 "wsrf_nw (wsrf_nw = wsrf_seq_nw)"="orange",
                 wsrf="red",
                 wsrf_t="red",
                 wsrf_c="blue",
                 randomForest="black", #"salmon",
                 wsrf_nw="orange",
                 wsrf_nw_sqrt="darkgreen",
                 wsrf_seq_nw="pink",
                 wsrf_seq_nw_sqrt="green",
                 bigrf="deeppink",
                 cforest="gray40")
  
  linelist <- c("wsrf (wsrf_c = wsrf_t)"=1,
                "wsrf_nw (wsrf_nw = wsrf_seq_nw)"=4,
                wsrf=1,
                wsrf_t=1,
                wsrf_c=2,
                randomForest=3,
                wsrf_nw=4,
                wsrf_nw_sqrt=5,
                wsrf_seq_nw=6,
                wsrf_seq_nw_sqrt=1,
                bigrf=2,
                cforest=3)

  shapelist <- c("wsrf (wsrf_c = wsrf_t)"=15,
                 "wsrf_nw (wsrf_nw = wsrf_seq_nw)"=11,
                 wsrf=15,
                 wsrf_t=15,
                 wsrf_c=4,
                 randomForest=17,
                 wsrf_nw=11,
                 wsrf_nw_sqrt=6,
                 wsrf_seq_nw=5,
                 wsrf_seq_nw_sqrt=8,
                 bigrf=16,
                 cforest=3)
  
  plfun <- function(measure, ylimt) {
    df <- datadf[!is.na(datadf[[measure]]), ]
    if (measure == "acc") {
      df <- subset(df, !modeller %in% c("wsrf_c",
                                        "wsrf_seq_nw",
                                        "wsrf_seq_nw_sqrt"))
      df$modeller[which(df$modeller=="wsrf_t")] <- "wsrf (wsrf_c = wsrf_t)"
      df$modeller[which(df$modeller=="wsrf_nw")] <- "wsrf_nw (wsrf_nw = wsrf_seq_nw)"
      ylimt <- c(0.663, 0.975)
      if (varname == "nvars")
        ylimt <- c(0.317, 0.975)
    } else if (measure == "elap") {
      df <- subset(df, !modeller %in% c("randomForest"))
    }
    p <- ggplot(df)
    p <- p + geom_line(aes_string(x=varname,
                                  y=measure,
                                  colour="modeller",
                                  linetype="modeller"))
    p <- p + geom_point(aes_string(x=varname,
                                   y=measure,
                                   colour="modeller",
                                   shape="modeller"))
    p <- p + scale_colour_manual("Modeller", values=colorlist)
    p <- p + scale_linetype_manual("Modeller", values=linelist)
    p <- p + scale_shape_manual("Modeller", values=shapelist)
    if (measure == "elap")
        p <- p + scale_y_continuous(labels=comma, trans="log10")

    if (facets && length(ylimt) < 2)
      p <- p + facet_wrap(~ ds, ncol=3, scales="free")
    else if (facets && length(ylimt) == 2){
      p <- p + ylim(ylimt[1], ylimt[2])
      p <- p + facet_wrap(~ ds, ncol=3)#, scales="free")
    }

    p <- p + ylab(switch(
      measure,
      acc="Accuracy",
      elap="Elapsed time (seconds)",
      mem="Used memory (MB)",
      varname
    ))

    p <- p + xlab(switch(
      varname,
      ntrees="Number of trees",
      nvars="Number of variables",
      nthreads="Number of threads",
      nnodes="Number of nodes",
      ""
    ))

    p <- p + theme(legend.position="top",
                   text=element_text(family="Times"),  # Using the default text font of JSS
                   legend.text=element_text(family="Courier"),  # For JSS \code
                   strip.text=element_text(family="Courier")
                   )

    return(p)
  }
  
  pls <- mapply(plfun, measures, ylimts, SIMPLIFY=FALSE)
  
  return(pls)
}

myutil$timeCvtFn <- function(x)
{
    # Format time in display.

    SECONDS <- c(s=1, m=60, h=3600, d=86400, y=31557600)
    unit <- ifelse(x < SECONDS[["m"]], "s",
            ifelse(x < SECONDS[["h"]], "m",
                    ifelse(x < SECONDS[["d"]], "h",
                            ifelse(x < SECONDS[["y"]], "d", "y"))))
    x <- round(x / SECONDS[unit], 0)
    x <- paste0(x, unit)
}

myutil$memCvtFn <- function(x)
{
    # Format memory in display.

    SIZES <- c(M=1, G=1024, T=1048576)
    unit <- ifelse(x < SIZES[["G"]], "M",
            ifelse(x < SIZES[["T"]], "G", "T"))
    
    x <- round(x / SIZES[unit], 0)
    x <- paste0(x, unit)
}

myutil$latexHighlightFn <- function(x,
                                    cvtFn, # format the value of x for better display
                                    iFn,   # find which values of x should be bold.
                                    jFn    # find which values of x should be underlined.
                                    ) {

    # Format the values of x and make corresponding hightlight in
    # LaTeX

    n <- names(x)
    i <- iFn(x)
    j <- jFn(x)
    x <- cvtFn(x)
    x[i] <- paste0("\\textbf{", x[i], "}")
    x[j] <- paste0("\\underline{", x[j], "}")
    names(x) <- n
    return(x)
}

myutil$extractRangeLatex <- function(
        results,  # experi results
        prefix,   # prefix of modellers to be extracted from results
        postfix,  # postfix of them
        label,    # label of modellers to be in plot legend
        varname,  # compared variables (such as ntrees, nvars, dsize)
        measure,  # evaluation measures (acc, cor, str, elap, mem)
        cvtfn
)
{
    # Extract the range of the values of measure (acc, elap, mem) for
    # each model.
    
    df <- myutil$extractdf(results, prefix, postfix, label, varname)
    df <- daply(df, .(ds, modeller), function(x) {
                measure_ranges <- c(min=min(x[[measure]]), max=max(x[[measure]]))
            })

    extractdf <- NULL
    for (e in dimnames(df)$ds) {
        n <- rownames(df[e, , ])
        if (measure == "acc") {
          x1 <- myutil$latexHighlightFn(df[e, , ][, 1], cvtfn, which.max, which.min)
          x2 <- myutil$latexHighlightFn(df[e, , ][, 2], cvtfn, which.max, which.min)
        } else {
          x1 <- myutil$latexHighlightFn(df[e, , ][, 1], cvtfn, which.min, which.max)
          x2 <- myutil$latexHighlightFn(df[e, , ][, 2], cvtfn, which.min, which.max)
        }
        x <- paste(x1, x2, sep="--")
        names(x) <- n
        extractdf <- rbind(extractdf, x)
    }
    rownames(extractdf) <- dimnames(df)$ds
    return(extractdf)
}

