######################################################################
# Code for generating Table 4.
#
# Summary for building a forest of trees whilst varying the number of
# variables.
######################################################################

# Load utils
source("../myutil.R")
source("../load.R")

# r holds all the results.
r <- experiSummary("../../../experiment-results/")

# for elapsed time
prefix <- c(paste0("^", "wsrf", "_100tv"),
            paste0("^", "wsrf", "_100cv"),
            paste0("^", "randomForest", "_100v"),
            paste0("^", "cforest", "_100v"),
            paste0("^", "wsrf", "_nw_100tv"),
            paste0("^", "wsrf", "_seq_nw_100tv"),
            paste0("^", "bigrfc", "_100v"))
postfix <- c("$",
             "$",
             "$",
             "$",
             "$",
             "$",
             "$")
label <- c("wsrf_t",
           "wsrf_c",
           "rf",
           "cforest",
           "wsrf_nw",
           "wsrf_seq_nw",
           "bigrf")
varname <- "nvars"
measure <- "elap"


# Extract results for Elaped time.
# Highlight and format elaps time in LaTeX.
#
# elaps will be a ndataset*nmodel matrix,
# where rows represent results for the datasets,
# column for the specific models, and the cell values are the 
# range of the elapsed time of varying the number of variables.
elaps <- myutil$extractRangeLatex(r,
                                  prefix,
                                  postfix,
                                  label,
                                  varname,
                                  measure,
                                  myutil$timeCvtFn)

# for accuracy
prefix <- c(paste0("^", "wsrf", "_100tv"),
            paste0("^", "randomForest", "_100v"),
            paste0("^", "cforest", "_100v"),
            paste0("^", "wsrf", "_nw_100tv"),
            paste0("^", "bigrfc", "_100v"))
postfix <- c("$",
             "$",
             "$",
             "$",
             "$")
label <- c("wsrf_t",
           "rf",
           "cforest",
           "wsrf_nw",
           "bigrf")
varname <- "nvars"
measure <- "acc"

# Extract results for Classification Accuracy.
# Highlight and format accuracy in LaTeX.
# 
# mems will be a ndataset*nmodel matrix,
# where rows represent results for the datasets,
# column for the specific models, and the cell values are the 
# range of accuracies of varying the number of variables.
accs <- myutil$extractRangeLatex(r,
                                 prefix,
                                 postfix,
                                 label,
                                 varname,
                                 measure,
                                 function(x){ round(100*x, 1) })

accs.colnames  <- colnames(accs)
accs           <- cbind(accs, rep("---", nrow(accs)), rep("---", nrow(accs)))
colnames(accs) <- c(accs.colnames, "wsrf_c", "wsrf_seq_nw")

# for memory
prefix <- c(paste0("^", "wsrf", "_100tv"),
            paste0("^", "randomForest", "_100v"),
            paste0("^", "cforest", "_100v"),
            paste0("^", "wsrf", "_nw_100tv"),
            paste0("^", "wsrf", "_seq_nw_100tv"),
            paste0("^", "bigrfc", "_100v"))
postfix <- c("$",
             "$",
             "$",
             "$",
             "$",
             "$")
label <- c("wsrf_t",
           "rf",
           "cforest",
           "wsrf_nw",
           "wsrf_seq_nw",
           "bigrf")
varname <- "nvars"
measure <- "mem"

# Extract results for Memory comsumption.
# Highlight and format memory in LaTeX.
# 
# mems will be a ndataset*nmodel matrix,
# where rows represent results for the datasets,
# column for the specific models, and the cell values are the 
# range of consumed memories of varying the number of variables.
mems <- myutil$extractRangeLatex(r,
                                 prefix,
                                 postfix,
                                 label,
                                 varname,
                                 measure,
                                 myutil$memCvtFn)

mems.colnames  <- colnames(mems)
mems           <- cbind(mems, rep("---", nrow(mems)))
colnames(mems) <- c(mems.colnames, "wsrf_c")

# Prepare LaTeX table.

rnames <- rownames(elaps)[ds.order]
rnames <- as.vector(sapply(rnames, function(e) {
                      e <- paste0("\\texttt{", e, "}")
                    }))

# Merge accs, elaps and mems into a bigger table.
col.order <- c("cforest", "randomForest", "bigrf", "wsrf_t", "wsrf_c", "wsrf_nw", "wsrf_seq_nw")
blanks <- rep(" ", length(col.order))

elaps <- elaps[ds.order, col.order]
elaps <- cbind(rnames, elaps)
elaps <- rbind(c("\\textit{Time}", blanks), elaps)

mems <- mems[ds.order, col.order]
mems <- cbind(rnames, mems)
mems <- rbind(c("\\textit{Memory}", blanks), mems)

accs <- accs[ds.order, col.order]
accs <- cbind(rnames, accs)
accs <- rbind(c("\\textit{Accuracy}", blanks), accs)

total <- rbind(accs, elaps, mems)

# Use model names as the first line of the table.
# 
# Escape "_" in LaTeX, and format model names.
cnames <- as.vector(sapply(col.order, function(e) {
                             e <- gsub("^([[:alpha:]]+)", "\\\\texttt{\\1}", e)
                             e <- gsub("_", "\\\\underline{{ }{ }}", e)
                           }))
total <- rbind(c(" ", cnames), total)

# Remove dimnames, otherwise, there will be warnings complain
# about duplicated rownames.
rownames(total) <- NULL
colnames(total) <- NULL

ali <- paste0("ll", paste0(rep("r", length(col.order)), collapse=""))
total <- xtable(total, 
                label="tab-numvar",
                caption=paste(
                  "Minimum and maximum accuracy, elapsed time and memory",
                  "used in building a forest of trees",
                  "whilst varying the number of variables (10, 20, 30, \\ldots, 100)",
                  "randomly selected at each node. ",
                  "The best results for each dataset are in boldface,",
                  "the worst are underlined. ",
                  "Since with the same set of random seeds",
                  "the results on accuracy of \\texttt{wsrf}\\underline{{ }{ }}c and \\texttt{wsrf}\\underline{{ }{ }}t are the same,",
                  "the results for \\texttt{wsrf}\\underline{{ }{ }}c are not presented, so are \\texttt{wsrf}\\underline{{ }{ }}seq\\underline{{ }{ }}nw."),
                align=ali)
print(total,
      caption.placement="bottom",
      sanitize.text.function=function(x){x},
      scalebox=0.72,
      hline.after=c(0, seq(from=1, by=nrow(elaps), length.out=4)),
      table.placement="h",
      include.colnames=FALSE,
      include.rownames=FALSE)
