######################################################################
# Code for generating Table 5.
#
# Summary for building a single tree.
######################################################################

# Load utils
source("../myutil.R")
source("../load.R")

# r holds all the results.
r <- experiSummary("../../../experiment-results/")

# Obtain results for building a single tree.
df <- subset(r, grepl("[[:alpha:]]+_b", modeller))

# Remove wsrf_nw_b, only keep wsrf_nw_sqrt_b.
df <- subset(df, modeller != "wsrf_nw_b")
df$modeller <- gsub("_b", "", df$modeller)
df$modeller <- gsub("bigrfc", "bigrf", df$modeller)
df$modeller <- gsub("wsrf_nw_sqrt", "wsrf_nw", df$modeller)
modellers <- unique(df$modeller)

# Extract results for Accuracy.
# accs will be a ndataset*nmodel matrix,
# where rows represent results for the datasets,
# column for the specific models, and the cell values are the 
# accuracies for building a single tree by the model.
accs <- daply(df, .(ds), function(m) 
            {
              t <- m$acc
              names(t) <- m$modeller
              a <- rep(NA, length(modellers))
              names(a) <- modellers
              for (x in m$modeller)
                a[x] <- t[x]
              return(a)
            })

# Highlight and format accuracy in LaTeX.
accs <- t(apply(accs, 1, myutil$latexHighlightFn,
              function(x){ round(100*x, 0) },
              which.max,
              which.min))


# Extract results for Elaped time.
# elaps will be a ndataset*nmodel matrix,
# where rows represent results for the datasets,
# column for the specific models, and the cell values are the 
# times for building a single tree by the model.
elaps <- daply(df, .(ds), function(m) 
             {
               t <- m$elap
               names(t) <- m$modeller
               a <- rep(NA, length(modellers))
               names(a) <- modellers
               for (x in m$modeller)
                 a[x] <- t[x]  
               return(a)
             })

# Highlight and format elaps time in LaTeX.
elaps <- t(apply(elaps, 1, myutil$latexHighlightFn,
               myutil$timeCvtFn,
               which.min,
               which.max))


# Extract results for Memory comsumption.
# mems will be a ndataset*nmodel matrix,
# where rows represent results for the datasets,
# column for the specific models, and the cell values are the 
# consumed memories for building a single tree by the model.
mems <- daply(df, .(ds), function(m) 
            {
              t <- m$mem
              names(t) <- m$modeller
              a <- rep(NA, length(modellers))
              names(a) <- modellers
              for (x in m$modeller)
                a[x] <- t[x]
              return(a)
            })

# Highlight and format memory in LaTeX.
mems <- t(apply(mems, 1, myutil$latexHighlightFn,
              myutil$memCvtFn,
              which.min,
              which.max))

# Prepare for LaTeX table.

rnames <- rownames(elaps)[ds.order]
rnames <- as.vector(sapply(rnames, function(e) {
                      e <- paste0("\\texttt{", e, "}")
                    }))

# Merge three measures into a bigger table.
col.order <- c("rpart", "cforest", "randomForest", "bigrf", "wsrf", "wsrf_nw")
blanks <- rep(" ", length(col.order))

elaps <- elaps[ds.order, col.order]
elaps <- cbind(rnames, elaps)
elaps <- rbind(c("\\textit{Time}", blanks), elaps)

mems <- mems[ds.order, col.order]
mems <- cbind(rnames, mems)
mems <- rbind(c("\\textit{Memory}", blanks), mems)

accs <- accs[ds.order, col.order]
accs <- cbind(rnames, accs)
accs <- rbind(c("\\textit{Accuracy}", blanks), accs)

total <- rbind(accs, elaps, mems)

# Use model names as the first line of the table.
# 
# Escape "_" in LaTeX, and format model names.
cnames <- as.vector(sapply(col.order, function(e) {
                             e <- gsub("^([[:alpha:]]+)", "\\\\texttt{\\1}", e)
                             e <- gsub("_", "\\\\underline{{ }{ }}", e)
                           }))
total <- rbind(c(" ", cnames), total)

# Remove dimnames, otherwise, there will be warnings complain
# about duplicated rownames.
rownames(total) <- NULL
colnames(total) <- NULL

ali <- paste0("ll", paste0(rep("r", length(col.order)), collapse=""))
total <- xtable(total, 
                label="tab-base-comp",
                caption=paste(
                  "Comparison of accuracy/time/memory for",
                  "a \\textit{\\textbf{single decision tree}} from the tree/forest",
                  "model building algorithms.",
                  "The time is in seconds, minutes, hours, and days",
                  "while accuracy is a percentage. ",
                  "The best results for each dataset are in boldface,",
                  "the worst are underlined."),
                align=ali)
print(total,
      caption.placement="bottom",
      sanitize.text.function=function(x){x},
      scalebox=0.72,
      hline.after=c(0, seq(from=1, by=nrow(elaps), length.out=4)),
      table.placement="h",
      include.colnames=FALSE,
      include.rownames=FALSE)
